% DESCRIPTION ber = cdma_test(users,cells,datalen,alamouti,sync,doplot)
%  Runs the simulation algorithm
% INPUT
%  users           the number of users in the system
%  cells           number of cells (1, 7 or 9)
%  datalen         number of data bits to transmit per mobile
%  alamouti        non-zero to activate Alamouti code
%  sync            non-zero to transmit in sync
%  doplot          non-zero to plot the user distribution
% OUTPUT
%  ber             a column vector with the ber of each link (mobile)
%
% $Id: cdma_test.m,v 1.29 2003-05-23 14:38:15 sharky Exp $

% Note : all binary data is given in bipolar (-1,1) format

function ber=cdma_test(i_nusers,i_cells,i_datalen,i_alamouti,i_sync,doplot)
par.seed=sum(clock);
setseed(par.seed);

% NUMBER OF USERS
par.ups = 64;      % maximum number of users per cell (sets channelization code length)
par.nusers = i_nusers;   % total number of users

% TRANMSISSION PARAMETERS
par.alamouti = i_alamouti;      % non-zero to use Alamouti code
par.data_len = i_datalen;       % number of bits of data
par.tc = 1;                    % chip duration (in samples)
par.tb = par.tc * par.ups;     % data bit duration (in samples)
if (i_sync)
  par.maxdelay = 0;            % users are syncrhonised
else
  par.maxdelay = par.tb;       % the users start transmittting at a random
                               % time of up to maxdelay (in samples)
end

                
% CELL AND PROPAGATION PARAMETERS                
par.alpha = 4;      % exponent of the distance in path loss calculation
par.sigma = 6;     % lognormal fading
par.cellradius = 1000;
switch (i_cells)
case(1)
  par.lm = 1;
  par.km = 0;
case(7)
  par.lm = 2;
  par.km = 1;
case(9)
  par.lm = 3;
  par.km = 0;
otherwise
  error('invalid number of cells, must be 1, 7 or 9');
end
par.sps = 1;
par.gainconst=-24;
par.raa=0.5;
par.corrdist=110;

% CODES AND WAVEFORM
par.pn = hadamard(par.ups);  % channelization codes
par.pulse = ones(1,par.tb);  % rectangular pulse


% place users in the cells
[ xym, xyb, fib, rombvec ] = network_configuration(par,doplot);
% allocate users to BTS's and channels to users
[ alloc.bts, alloc.channel ] = cdma_alloc(xym,xyb,par);


% TRANSMISSION
[trans,h0,h1] = cdma_transmit(par,alloc);


% CHANNEL
% path loss + shadowing
gm=channel_model(xym,xyb,fib,rombvec,par);
% power control : compensate path loss to *our* BTS
trans_power = -gm((1:par.nusers) + (alloc.bts(1:par.nusers)'-1)*par.nusers);
if (size(trans_power) == size(gm))
  trans_power = trans_power';
end
% calculate received signal as base stations
recv_spread =  db2lin( ones(size(xyb,2),1) * trans_power + gm') * trans.signal;


% RECEPTION
recv_data = cdma_receive(recv_spread,trans.delay,par,alloc,h0,h1);

% OUTPUT RESULTS
nerrors = sum(recv_data ~= trans.data,2);
ber = 100 * nerrors / par.data_len;

